/**
 *
 */
package world.bentobox.bentobox.listeners.flags;

import java.util.LinkedList;
import java.util.Queue;

import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.World.Environment;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.world.ChunkLoadEvent;
import org.bukkit.scheduler.BukkitTask;

import world.bentobox.bentobox.BentoBox;
import world.bentobox.bentobox.api.events.BentoBoxReadyEvent;
import world.bentobox.bentobox.api.flags.FlagListener;
import world.bentobox.bentobox.lists.Flags;
import world.bentobox.bentobox.util.Pair;

/**
 * Cleans super-flat world chunks or normal nether chunks if they generate accidentally
 * due to lack of a generator being loaded
 * @author tastybento
 *
 */
public class CleanSuperFlatListener extends FlagListener {

    private Queue<Pair<Integer, Integer>> chunkQ = new LinkedList<>();
    private BukkitTask task;
    private boolean ready;

    @EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
    public void onBentoBoxReady(BentoBoxReadyEvent e) {
        ready = true;
    }

    @SuppressWarnings("deprecation")
    @EventHandler(priority = EventPriority.LOW, ignoreCancelled = true)
    public void onChunkLoad(ChunkLoadEvent e) {
        if (!ready) {
            return;
        }
        BentoBox plugin = BentoBox.getInstance();
        World world = e.getWorld();
        if (!e.getChunk().getBlock(0, 0, 0).getType().equals(Material.BEDROCK)
                || !Flags.CLEAN_SUPER_FLAT.isSetForWorld(world)
                || (world.getEnvironment().equals(Environment.NETHER) && (!plugin.getIWM().isNetherGenerate(world) || !plugin.getIWM().isNetherIslands(world)))
                || (world.getEnvironment().equals(Environment.THE_END) && (!plugin.getIWM().isEndGenerate(world) || !plugin.getIWM().isEndIslands(world)))) {
            return;
        }
        // Add to queue
        chunkQ.add(new Pair<>(e.getChunk().getX(), e.getChunk().getZ()));
        if (task == null) {
            task = Bukkit.getScheduler().runTaskTimer(plugin, () -> {
                if (!chunkQ.isEmpty()) {
                    Pair<Integer, Integer> chunkXZ = chunkQ.poll();
                    world.regenerateChunk(chunkXZ.x, chunkXZ.z); // NOSONAR - the deprecation doesn't the cause any issues to us
                    plugin.logWarning(chunkQ.size() + " Regenerating superflat chunk " + world.getName() + " " + chunkXZ.x + ", " + chunkXZ.z);
                }
            }, 0L, 1L);
        }
    }
}
